#!/bin/bash
#
##
# clpawssip-start.sh
##
# start the clpawssip
##


echo "clpawssip start script start."


##
# ret code
##
AWSSIP_SUCCESS_CODE=0					# succeed
AWSSIP_ERR_SCHAWSCLI_CODE=50			# failed to search aws cli path
AWSSIP_ERR_GETOCFENV_CODE=51			# failed to get the ocf environment variable
AWSSIP_ERR_AWSASSIGNIP_CODE=52			# failed to assign ip address for aws
# AWSSIP_ERR_AWSUNASSIGNIP_CODE=53		# failed to unassign ip address for aws
AWSSIP_ERR_AWSCHKIP_CODE=54				# failed to check ip address for aws
AWSSIP_ERR_OSASSIGNIP_CODE=55			# failed to assign ip address for os
# AWSSIP_ERR_OSUNASSIGNIP_CODE=56		# failed to unassign ip address for os
AWSSIP_ERR_OSCHKIP_CODE=57				# failed to check ip address for os
AWSSIP_ERR_GETMAC_CODE=58				# failed to get mac address
AWSSIP_ERR_GETNICNAME_CODE=59			# failed to get nic name
AWSSIP_ERR_GETSUBNET_CODE=60			# failed to get subnet id
AWSSIP_ERR_GETCIDR_CODE=61				# failed to get cidrblock


##
# resultmsg
##
AWSSIP_ERR_SCHAWSCLI_MSG="The AWS CLI command is not found."
AWSSIP_ERR_GETOCFENV_MSG="Failed to obtain the setting value."
AWSSIP_ERR_AWSASSIGNIP_MSG="Failed to assign the secondary IP address on the AWS side."
# AWSSIP_ERR_AWSUNASSIGNIP_MSG="Failed to release the assigned secondary IP address on the AWS side."
AWSSIP_ERR_AWSCHKIP_MSG="Failed to process checking the secondary IP address on the AWS side."
AWSSIP_ERR_OSASSIGNIP_MSG="Failed to assign the secondary IP address on the OS side."
# AWSSIP_ERR_OSUNASSIGNIP_MSG="Failed to release the assigned secondary IP address on the OS side."
AWSSIP_ERR_OSCHKIP_MSG="Failed to process checking the secondary IP address on the OS side."
AWSSIP_ERR_GETMAC_MSG="Failed to obtain a MAC address."
AWSSIP_ERR_GETNICNAME_MSG="Failed to obtain a NIC name."
AWSSIP_ERR_GETSUBNET_MSG="Failed to obtain a subnet ID."
AWSSIP_ERR_GETCIDR_MSG="Failed to obtain a CIDR block."


##
# set the value to use for the shmrmset command
##
GROUP_NAME="${CLP_GROUPNAME}"				# group name
RESOURCE_NAME="${CLP_RESOURCENAME}"		# resource name
RESOURCE_TYPE="awssip"						# resource type

if [ -z "${GROUP_NAME}" ] || [ -z "${RESOURCE_NAME}" ]
then
	echo "failed to obtain the value required for the shmrmset command." 1>&2
fi


##
# awscli path
##
export PATH=$PATH:/usr/local/bin


##
# awscli path search
##
which aws > /dev/null 2>&1

if [ $? -ne 0 ]
then
	clpshmrmset --descript --rsc -g "${GROUP_NAME}" -t "${RESOURCE_TYPE}" -n "${RESOURCE_NAME}" -m "${AWSSIP_ERR_SCHAWSCLI_MSG}" > /dev/null 2>&1
	echo "failed to search aws cli path." 1>&2
	echo "clpawssip start script has failed." 1>&2
	exit ${AWSSIP_ERR_SCHAWSCLI_CODE}
fi


##
# set environment variables for use with the aws cli
##
CURRENT=$(cd $(dirname $0);pwd)

IFS=$'\n'

for ROW in `cat ${CURRENT}/clpaws_setting.conf`
do
	if [[ "${ROW}" =~ "=" && ! "${ROW}" =~ "#" ]]
	then
		LEFT=`expr index "${ROW}" "="`
		LEFT=$((LEFT - 1))
		ENV_NAME="`echo \"${ROW}\" | cut -c -${LEFT}`"
		ENV_NAME="`echo \"${ENV_NAME}\" | sed -e 's/[ \t]*$//'`"
		
		RIGHT=`expr index "${ROW}" "="`
		RIGHT=$((RIGHT + 1))
		ENV_VALUE="`echo \"${ROW}\" | cut -c ${RIGHT}-`"
		ENV_VALUE="`echo \"${ENV_VALUE}\" | sed 's/^[ \t]*//'`"
		
		export "`echo \"${ENV_NAME}\"`"="`echo \"${ENV_VALUE}\"`"
	fi
done

unset IFS


##
# set ocf environment variable
##
SECONDARY_IP="${CLP_OCF_PARAM1}"		# secondary ip address
ENI_ID="${CLP_OCF_PARAM2}"			# ENI ID
PING_TIMEOUT="${CLP_OCF_PARAM3}"		# ping timeout

if [ -z "${SECONDARY_IP}" ] || [ -z "${ENI_ID}" ] || [ -z "${PING_TIMEOUT}" ]
then
	clpshmrmset --descript --rsc -g "${GROUP_NAME}" -t "${RESOURCE_TYPE}" -n "${RESOURCE_NAME}" -m "${AWSSIP_ERR_GETOCFENV_MSG}" > /dev/null 2>&1
	echo "failed to get the ocf environment variable." 1>&2
	echo "clpawssip start script has failed." 1>&2
	exit ${AWSSIP_ERR_GETOCFENV_CODE}
fi


##
# determine whether the secondary IP address is ipv4 or ipv6
##
if [ `echo "${SECONDARY_IP}" | grep :` ]
then
	SECONDARYIP_TYPE="IPv6"
	echo "secondary ip address is ipv6."
else
	SECONDARYIP_TYPE="IPv4"
	echo "secondary ip address is ipv4."
fi


##
# check secondary ip exists in aws
##
if [ "${SECONDARYIP_TYPE}" == "IPv6" ]
then
	AWS_CHECK_IP="`aws ec2 describe-network-interfaces --network-interface-ids \"${ENI_ID}\" \
							--filters \"Name=ipv6-addresses.ipv6-address,Values=\"${SECONDARY_IP}\"\" --output text`"
elif [ "${SECONDARYIP_TYPE}" == "IPv4" ]
then
	AWS_CHECK_IP="`aws ec2 describe-network-interfaces --network-interface-ids \"${ENI_ID}\" \
						--filters \"Name=addresses.private-ip-address,Values=\"${SECONDARY_IP}\"\" --output text`"
fi

if [ $? -eq 0 ] && [ -n "${AWS_CHECK_IP}" ]
then
	AWS_SECONDARYIP="exists"
else
	AWS_SECONDARYIP="not exists"
fi


##
#
# execute when the secondary ip does not exist on the aws side
#
##
if [ "${AWS_SECONDARYIP}" == "exists" ]
then
	echo "skip the process because the secondary ip address of aws exists."
else


##
# assign a secondary ip to aws
##
	if [ "${SECONDARYIP_TYPE}" == "IPv6" ]
	then
		aws ec2 assign-ipv6-addresses --network-interface-id "${ENI_ID}" --ipv6-addresses "${SECONDARY_IP}" --output text
	elif [ "${SECONDARYIP_TYPE}" == "IPv4" ]
	then
		aws ec2 assign-private-ip-addresses --allow-reassignment --network-interface-id "${ENI_ID}" --private-ip-addresses "${SECONDARY_IP}" --output text
	fi
	
	if [ $? -ne 0 ]
	then
		clpshmrmset --descript --rsc -g "${GROUP_NAME}" -t "${RESOURCE_TYPE}" -n "${RESOURCE_NAME}" -m "${AWSSIP_ERR_AWSASSIGNIP_MSG}" > /dev/null 2>&1
		echo "failed to assign ip address for aws." 1>&2
		echo "clpawssip start script has failed." 1>&2
		exit ${AWSSIP_ERR_AWSASSIGNIP_CODE}
	fi


##
# check secondary ip exists in aws
##
	if [ "${SECONDARYIP_TYPE}" == "IPv6" ]
	then
		AWS_CHECK_IP="`aws ec2 describe-network-interfaces --network-interface-ids \"${ENI_ID}\" \
								--filters \"Name=ipv6-addresses.ipv6-address,Values=\"${SECONDARY_IP}\"\" --output text`"
	elif [ "${SECONDARYIP_TYPE}" == "IPv4" ]
	then
		AWS_CHECK_IP="`aws ec2 describe-network-interfaces --network-interface-ids \"${ENI_ID}\" \
							--filters \"Name=addresses.private-ip-address,Values=\"${SECONDARY_IP}\"\" --output text`"
	fi
	
	if [ $? -ne 0 ] || [ -z "${AWS_CHECK_IP}" ]
	then
		clpshmrmset --descript --rsc -g "${GROUP_NAME}" -t "${RESOURCE_TYPE}" -n "${RESOURCE_NAME}" -m "${AWSSIP_ERR_AWSCHKIP_MSG}" > /dev/null 2>&1
		echo "failed to check the existence of ip address of aws." 1>&2
		echo "clpawssip start script has failed." 1>&2
		exit ${AWSSIP_ERR_AWSCHKIP_CODE}
	fi
fi


##
# check secondary ip exists in os
##
clpping -c ${PING_TIMEOUT} "${SECONDARY_IP}" > /dev/null 2>&1
if [ $? -eq 0 ]
then
	OS_SECONDARYIP="exists"
else
	OS_SECONDARYIP="not exists"
fi


##
# If a secondary IP exists on both the AWS side and the OS side in the pre-check, it terminates normally
##
if [ "${AWS_SECONDARYIP}" == "exists" ] && [ "${OS_SECONDARYIP}" == "exists" ]
then
	echo "skip the process because the secondary ip address of os exists."
	echo "secondary ip address is exists."
	echo "clpawssip start script has succeeded."
	exit ${AWSSIP_SUCCESS_CODE}
fi


##
#
# execute when the secondary ip does not exist on the os side
#
##
if [ "${OS_SECONDARYIP}" == "exists" ]
then
	echo "skip the process because the secondary ip address of os exists."
	echo "clpawssip start script has succeeded."
	exit ${AWSSIP_SUCCESS_CODE}
fi


##
# nic name acquisition process
##
MAC_ADDRESS="`aws ec2 describe-network-interfaces --network-interface-ids \"${ENI_ID}\" \
				--query \"NetworkInterfaces[].[MacAddress]\" --output text`"
if [ $? -ne 0 ]
then
	if [ "${SECONDARYIP_TYPE}" == "IPv6" ]
	then
		aws ec2 unassign-ipv6-addresses --network-interface-id "${ENI_ID}" --ipv6-addresses "${SECONDARY_IP}" > /dev/null 2>&1
	elif [ "${SECONDARYIP_TYPE}" == "IPv4" ]
	then
		aws ec2 unassign-private-ip-addresses --network-interface-id "${ENI_ID}" --private-ip-addresses "${SECONDARY_IP}" > /dev/null 2>&1
	fi
	clpshmrmset --descript --rsc -g "${GROUP_NAME}" -t "${RESOURCE_TYPE}" -n "${RESOURCE_NAME}" -m "${AWSSIP_ERR_GETMAC_MSG}" > /dev/null 2>&1
	echo "failed to get mac address." 1>&2
	echo "clpawssip start script has failed." 1>&2
	exit ${AWSSIP_ERR_GETMAC_CODE}
fi

NIC_NAME="`ip link | awk '/^[0-9]/ { iface=$2 } /link/ { mac=$2 } /brd/ { print iface, mac }' | grep \"${MAC_ADDRESS}\" | awk '{ sub(\":\", \"\") }  { print $1 }'`"
if [ $? -ne 0 ] || [ -z "${NIC_NAME}" ]
then
	if [ "${SECONDARYIP_TYPE}" == "IPv6" ]
	then
		aws ec2 unassign-ipv6-addresses --network-interface-id "${ENI_ID}" --ipv6-addresses "${SECONDARY_IP}" > /dev/null 2>&1
	elif [ "${SECONDARYIP_TYPE}" == "IPv4" ]
	then
		aws ec2 unassign-private-ip-addresses --network-interface-id "${ENI_ID}" --private-ip-addresses "${SECONDARY_IP}" > /dev/null 2>&1
	fi
	clpshmrmset --descript --rsc -g "${GROUP_NAME}" -t "${RESOURCE_TYPE}" -n "${RESOURCE_NAME}" -m "${AWSSIP_ERR_GETNICNAME_MSG}" > /dev/null 2>&1
	echo "failed to get nic name (mac=${MAC_ADDRESS})." 1>&2
	echo "clpawssip start script has failed." 1>&2
	exit ${AWSSIP_ERR_GETNICNAME_CODE}
fi

echo "mac=${MAC_ADDRESS} nic=${NIC_NAME}"


##
# prefix acquisition process
##
SUBNET_ID="`aws ec2 describe-network-interfaces --network-interface-ids \"${ENI_ID}\" \
				--query \"NetworkInterfaces[].[SubnetId]\" --output text`"
if [ $? -ne 0 ]
then
	if [ "${SECONDARYIP_TYPE}" == "IPv6" ]
	then
		aws ec2 unassign-ipv6-addresses --network-interface-id "${ENI_ID}" --ipv6-addresses "${SECONDARY_IP}" > /dev/null 2>&1
	elif [ "${SECONDARYIP_TYPE}" == "IPv4" ]
	then
		aws ec2 unassign-private-ip-addresses --network-interface-id "${ENI_ID}" --private-ip-addresses "${SECONDARY_IP}" > /dev/null 2>&1
	fi
	clpshmrmset --descript --rsc -g "${GROUP_NAME}" -t "${RESOURCE_TYPE}" -n "${RESOURCE_NAME}" -m "${AWSSIP_ERR_GETSUBNET_MSG}" > /dev/null 2>&1
	echo "failed to get subnet id." 1>&2
	echo "clpawssip start script has failed." 1>&2
	exit ${AWSSIP_ERR_GETSUBNET_CODE}
fi


if [ "${SECONDARYIP_TYPE}" == "IPv6" ]
then
	CIDR="`aws ec2 describe-subnets --subnet-ids \"${SUBNET_ID}\" \
				--query \"Subnets[].Ipv6CidrBlockAssociationSet[].[Ipv6CidrBlock]\" --output text`"
elif [ "${SECONDARYIP_TYPE}" == "IPv4" ]
then
	CIDR="`aws ec2 describe-subnets --subnet-ids \"${SUBNET_ID}\" \
				--query \"Subnets[].[CidrBlock]\" --output text`"
fi

if [ $? -ne 0 ]
then
	if [ "${SECONDARYIP_TYPE}" == "IPv6" ]
	then
		aws ec2 unassign-ipv6-addresses --network-interface-id "${ENI_ID}" --ipv6-addresses "${SECONDARY_IP}" > /dev/null 2>&1
	elif [ "${SECONDARYIP_TYPE}" == "IPv4" ]
	then
		aws ec2 unassign-private-ip-addresses --network-interface-id "${ENI_ID}" --private-ip-addresses "${SECONDARY_IP}" > /dev/null 2>&1
	fi
	clpshmrmset --descript --rsc -g "${GROUP_NAME}" -t "${RESOURCE_TYPE}" -n "${RESOURCE_NAME}" -m "${AWSSIP_ERR_GETCIDR_MSG}" > /dev/null 2>&1
	echo "failed to get the cidrblock of ip address (subnet=${SUBNET_ID})." 1>&2
	echo "clpawssip start script has failed." 1>&2
	exit ${AWSSIP_ERR_GETCIDR_CODE}
fi


PREFIX="`echo ${CIDR##*/}`"

echo "subnet=${SUBNET_ID} cidr=${CIDR} prefix=${PREFIX}"


##
# assign a secondary ip to os
##
ip address add "${SECONDARY_IP}"/"${PREFIX}" dev "${NIC_NAME}"
if [ $? -ne 0 ]
then
	if [ "${SECONDARYIP_TYPE}" == "IPv6" ]
	then
		aws ec2 unassign-ipv6-addresses --network-interface-id "${ENI_ID}" --ipv6-addresses "${SECONDARY_IP}" > /dev/null 2>&1
	elif [ "${SECONDARYIP_TYPE}" == "IPv4" ]
	then
		aws ec2 unassign-private-ip-addresses --network-interface-id "${ENI_ID}" --private-ip-addresses "${SECONDARY_IP}" > /dev/null 2>&1
	fi
	clpshmrmset --descript --rsc -g "${GROUP_NAME}" -t "${RESOURCE_TYPE}" -n "${RESOURCE_NAME}" -m "${AWSSIP_ERR_OSASSIGNIP_MSG}" > /dev/null 2>&1
	echo "failed to assign ip address for os." 1>&2
	echo "clpawssip start script has failed." 1>&2
	exit ${AWSSIP_ERR_OSASSIGNIP_CODE}
fi


##
# check secondary ip exists in os
##
clpping -c ${PING_TIMEOUT} "${SECONDARY_IP}" > /dev/null 2>&1
if [ $? -ne 0 ]
then
	if [ "${SECONDARYIP_TYPE}" == "IPv6" ]
	then
		aws ec2 unassign-ipv6-addresses --network-interface-id "${ENI_ID}" --ipv6-addresses "${SECONDARY_IP}" > /dev/null 2>&1
	elif [ "${SECONDARYIP_TYPE}" == "IPv4" ]
	then
		aws ec2 unassign-private-ip-addresses --network-interface-id "${ENI_ID}" --private-ip-addresses "${SECONDARY_IP}" > /dev/null 2>&1
	fi
	clpshmrmset --descript --rsc -g "${GROUP_NAME}" -t "${RESOURCE_TYPE}" -n "${RESOURCE_NAME}" -m "${AWSSIP_ERR_OSCHKIP_MSG}" > /dev/null 2>&1
	echo "failed to check ip address for os." 1>&2
	echo "clpawssip start script has failed." 1>&2
	exit ${AWSSIP_ERR_OSCHKIP_CODE}
fi


##
# end
##
echo "clpawssip start script has succeeded."

exit ${AWSSIP_SUCCESS_CODE}

